% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rhierMnlRwMixtureParallel.R
\name{rhierMnlRwMixtureParallel}
\alias{rhierMnlRwMixtureParallel}
\title{MCMC Algorithm for Hierarchical Multinomial Logit with Mixture-of-Normals Heterogeneity}
\usage{
rhierMnlRwMixtureParallel(Data, Prior, Mcmc, verbose = FALSE)
}
\arguments{
\item{Data}{A list containing: `p`- number of choice alternatives, `lgtdata` - A \eqn{nlgt} size list of multinomial logistic data, and optional `Z`- matrix of unit characteristics.}

\item{Prior}{A list with one required parameter: `ncomp`, and optional parameters: `mubar`, `Amu`, `nu`, `V`, `Ad`, `deltaBar`, and `a`.}

\item{Mcmc}{A list with one required parameter: 'R' - number of iterations, and optional parameters: `s`, `w`, `keep`, `nprint`, and `drawcomp`.}

\item{verbose}{If \code{TRUE}, enumerates model parameters and timing information.}
}
\value{
A list of sharded partitions where each partition contains the following:
\item{compdraw}{A list (length: R/keep) where each list contains `mu` (vector, length: `ncomp`) and `rooti` (matrix, shape: ncomp \eqn{\times} ncomp)}
\item{probdraw}{A \eqn{(R/keep) \times (ncomp)} matrix that reports the probability that each draw came from a particular component}
\item{Deltadraw}{A \eqn{(R/keep) \times (nz \times nvar)} matrix of draws of Delta, first row is initial value. This could be null if Z is NULL}
}
\description{
\code{rhierMnlRwMixtureParallel} implements a MCMC algorithm for a hierarchical multinomial logit model with a mixture of normals heterogeneity distribution.
}
\details{
\subsection{Model and Priors}{
\eqn{y_i} \eqn{\sim}{~} \eqn{MNL(X_i,\beta_i)} with \eqn{i = 1, \ldots,} length(lgtdata) 
and where \eqn{\beta_i} is \eqn{1 \times nvar}

\eqn{\beta_i} = \eqn{Z\Delta}[i,] + \eqn{u_i} \cr
Note:  Z\eqn{\Delta} is the matrix Z \eqn{ \times \Delta} and [i,] refers to \eqn{i}th row of this product \cr
Delta is an \eqn{nz \times nvar} array 

\eqn{u_i} \eqn{\sim}{~} \eqn{N(\mu_{ind},\Sigma_{ind})} with \eqn{ind} \eqn{\sim}{~} multinomial(pvec)

\eqn{pvec}                \eqn{\sim}{~} dirichlet(a) \cr
\eqn{delta = vec(\Delta)} \eqn{\sim}{~} \eqn{N(deltabar, A_d^{-1})} \cr
\eqn{\mu_j}               \eqn{\sim}{~} \eqn{N(mubar, \Sigma_j (x) Amu^{-1})} \cr
\eqn{\Sigma_j}            \eqn{\sim}{~} \eqn{IW(nu, V)}

Note: \eqn{Z} should NOT include an intercept and is centered for ease of interpretation. 
The mean of each of the \code{nlgt} \eqn{\beta}s is the mean of the normal mixture. 
Use \code{summary()} to compute this mean from the \code{compdraw} output.\cr

Be careful in assessing prior parameter \code{Amu}: 0.01 is too small for many applications. 
See chapter 5 of Rossi et al for full discussion.
}
\subsection{Argument Details}{
\emph{\code{Data  = list(lgtdata, Z, p)} [\code{Z} optional]}
\tabular{ll}{
\code{lgtdata:        } \tab A \eqn{nlgt} size list of multinominal lgtdata \cr
\code{lgtdata[[i]]$y: } \tab \eqn{n_i \times 1} vector of multinomial outcomes (1, \ldots, p) for \eqn{i}th unit\cr
\code{lgtdata[[i]]$X: } \tab \eqn{(n_i \times p) \times nvar} design matrix for \eqn{i}th unit \cr
\code{Z:              } \tab An \eqn{(nlgt) \times nz} matrix of unit characteristics\cr 
\code{p:              } \tab number of choice alternatives 
}
\emph{\code{Prior = list(a, deltabar, Ad, mubar, Amu, nu, V, a, ncomp)} [all but \code{ncomp} are optional]}
\tabular{ll}{
\code{a:              } \tab \eqn{ncomp \times 1} vector of Dirichlet prior parameters (def: \code{rep(5,ncomp)}) \cr
\code{deltabar:       } \tab \eqn{(nz \times nvar) \times 1} vector of prior means (def: 0) \cr
\code{Ad:             } \tab prior precision matrix for vec(D) (def: 0.01*I) \cr
\code{mubar:          } \tab \eqn{nvar \times 1} prior mean vector for normal component mean (def: 0 if unrestricted; 2 if restricted) \cr
\code{Amu:            } \tab prior precision for normal component mean (def: 0.01 if unrestricted; 0.1 if restricted) \cr
\code{nu:             } \tab d.f. parameter for IW prior on normal component Sigma (def: nvar+3 if unrestricted; nvar+15 if restricted) \cr
\code{V:              } \tab PDS location parameter for IW prior on normal component Sigma (def: nu*I if unrestricted; nu*D if restricted with d_pp = 4 if unrestricted and d_pp = 0.01 if restricted) \cr
\code{ncomp:          } \tab number of components used in normal mixture 
}
\emph{\code{Mcmc  = list(R, keep, nprint, s, w)} [only \code{R} required]}
\tabular{ll}{
\code{R:              } \tab number of MCMC draws \cr
\code{keep:           } \tab MCMC thinning parameter -- keep every \code{keep}th draw (def: 1) \cr
\code{nprint:         } \tab print the estimated time remaining for every \code{nprint}'th draw (def: 100, set to 0 for no print) \cr
\code{s:              } \tab scaling parameter for RW Metropolis (def: 2.93/\code{sqrt(nvar)}) \cr
\code{w:              } \tab fractional likelihood weighting parameter (def: 0.1)
}
}
}
\examples{

\donttest{

R=500

######### Single Component with rhierMnlRwMixtureParallel########
##parameters
p = 3 # num of choice alterns                            
ncoef = 3  
nlgt = 1000                          
nz = 2
Z = matrix(runif(nz*nlgt),ncol=nz)
Z = t(t(Z)-apply(Z,2,mean)) # demean Z

ncomp = 1 # no of mixture components
Delta = matrix(c(1,0,1,0,1,2),ncol=2)
comps = NULL
comps[[1]] = list(mu=c(0,2,1),rooti=diag(rep(1,3)))
pvec = c(1)

simmnlwX = function(n,X,beta){
  k=length(beta)
  Xbeta=X \%*\% beta
  j=nrow(Xbeta)/n
  Xbeta=matrix(Xbeta,byrow=TRUE,ncol=j)
  Prob=exp(Xbeta)
  iota=c(rep(1,j))
  denom=Prob \%*\% iota
  Prob=Prob/as.vector(denom)
  y=vector("double",n)
  ind=1:j
  for (i in 1:n) { 
    yvec = rmultinom(1, 1, Prob[i,])
    y[i] = ind\%*\%yvec
  }
  return(list(y=y,X=X,beta=beta,prob=Prob))
}

## simulate data
simlgtdata=NULL
ni=rep(5,nlgt) 
for (i in 1:nlgt) 
{
  if (is.null(Z))
  {
    betai=as.vector(bayesm::rmixture(1,pvec,comps)$x)
  } else {
    betai=Delta \%*\% Z[i,]+as.vector(bayesm::rmixture(1,pvec,comps)$x)
  }
  Xa=matrix(runif(ni[i]*p,min=-1.5,max=0),ncol=p)
  X=bayesm::createX(p,na=1,nd=NULL,Xa=Xa,Xd=NULL,base=1)
  outa=simmnlwX(ni[i],X,betai)
  simlgtdata[[i]]=list(y=outa$y,X=X,beta=betai)
}

## set MCMC parameters
Prior1=list(ncomp=ncomp) 
keep=1
Mcmc1=list(R=R,keep=keep) 
Data1=list(list(p=p,lgtdata=simlgtdata,Z=Z))
s = 1
Data2 = partition_data(Data1, s=s)

out2 = parallel::mclapply(Data2, FUN = rhierMnlRwMixtureParallel, Prior = Prior1,
Mcmc = Mcmc1, mc.cores = s, mc.set.seed = FALSE)

######### Multiple Components with rhierMnlRwMixtureParallel########
##parameters
R = 500
p=3 # num of choice alterns                            
ncoef=3  
nlgt=1000  # num of cross sectional units                         
nz=2
Z=matrix(runif(nz*nlgt),ncol=nz)
Z=t(t(Z)-apply(Z,2,mean)) # demean Z

ncomp=3
Delta=matrix(c(1,0,1,0,1,2),ncol=2) 

comps=NULL 
comps[[1]]=list(mu=c(0,2,1),rooti=diag(rep(1,3)))
comps[[2]]=list(mu=c(1,0,2),rooti=diag(rep(1,3)))
comps[[3]]=list(mu=c(2,1,0),rooti=diag(rep(1,3)))
pvec=c(.4,.2,.4)

simmnlwX= function(n,X,beta) {
  k=length(beta)
  Xbeta=X \%*\% beta
  j=nrow(Xbeta)/n
  Xbeta=matrix(Xbeta,byrow=TRUE,ncol=j)
  Prob=exp(Xbeta)
  iota=c(rep(1,j))
  denom=Prob \%*\% iota
  Prob=Prob/as.vector(denom)
  y=vector("double",n)
  ind=1:j
  for (i in 1:n) 
  {yvec=rmultinom(1,1,Prob[i,]); y[i]=ind \%*\% yvec}
  return(list(y=y,X=X,beta=beta,prob=Prob))
}

## simulate data
simlgtdata=NULL
ni=rep(5,nlgt) 
for (i in 1:nlgt) 
{
  if (is.null(Z))
  {
    betai=as.vector(bayesm::rmixture(1,pvec,comps)$x)
  } else {
    betai=Delta \%*\% Z[i,]+as.vector(bayesm::rmixture(1,pvec,comps)$x)
  }
  Xa=matrix(runif(ni[i]*p,min=-1.5,max=0),ncol=p)
  X=bayesm::createX(p,na=1,nd=NULL,Xa=Xa,Xd=NULL,base=1)
  outa=simmnlwX(ni[i],X,betai) 
  simlgtdata[[i]]=list(y=outa$y,X=X,beta=betai)
}

## set parameters for priors and Z
Prior1=list(ncomp=ncomp) 
keep=1
Mcmc1=list(R=R,keep=keep) 
Data1=list(list(p=p,lgtdata=simlgtdata,Z=Z))
s = 1
Data2 = partition_data(Data1,s)

out2 = parallel::mclapply(Data2, FUN = rhierMnlRwMixtureParallel, Prior = Prior1,
Mcmc = Mcmc1, mc.cores = s, mc.set.seed = FALSE)
}


}
\references{
Bumbaca, Federico (Rico), Sanjog Misra, and Peter E. Rossi (2020), "Scalable Target Marketing:
Distributed Markov Chain Monte Carlo for Bayesian Hierarchical Models", Journal of Marketing
Research, 57(6), 999-1018.

Chapter 5, \emph{Bayesian Statistics and Marketing} by Rossi, Allenby, and McCulloch.
}
\seealso{
\code{\link{partition_data}}, 
\code{\link{drawPosteriorParallel}}, 
\code{\link{combine_draws}}, 
\code{\link{rheteroMnlIndepMetrop}}
}
\author{
Federico Bumbaca, Leeds School of Business, University of Colorado Boulder, \email{federico.bumbaca@colorado.edu}
}
